@@ -3,7 +3,7 @@
 
 pyver=$(python -c 'import sys; print("%s%s" % sys.version_info[:2])')
 ver=$(python setup.py --version)
-pkg=fastavro-${ver}-py${pyver}_0.tar.bz
+pkg=fastavro-${ver}-py${pyver}_0.tar.bz2
 
 
 # Fail on 1'st error
@@ -11,12 +11,13 @@ set -e
 set -x
 
 make
-python3 setup.py bdist_egg upload
-python2 setup.py bdist_egg upload
-python setup.py sdist upload
-/opt/anaconda/bin/python setup.py bdist_conda
-/opt/anaconda/bin/anaconda upload /opt/anaconda/conda-bld/linux-64/${pkg}
-rm -fr build dist fastavro.egg-info/
+python setup.py sdist
+twine upload dist/fastavro-${ver}.tar.gz
+conda build .
+anaconda upload /opt/anaconda3/conda-bld/linux-64/${pkg}
 git tag -f $(python setup.py --version)
 git push
 git push --tags
+# print sha so we can use it in conda-forge recipe
+sha256sum dist/fastavro-${ver}.tar.gz
+rm -fr build dist fastavro.egg-info/
