@@ -4,6 +4,9 @@ from fastavro.six import MemoryIO
 from os.path import join, abspath, dirname, basename
 from glob import iglob
 
+from nose import SkipTest
+from nose.tools import raises
+
 data_dir = join(abspath(dirname(__file__)), 'avro-files')
 
 try:
@@ -72,13 +75,10 @@ def test_fastavro():
         yield check, filename
 
 
+@raises(ValueError)
 def test_not_avro():
-    try:
-        with open(__file__, 'rb') as fo:
-            fastavro.reader(fo)
-        assert False, 'opened non avro file'
-    except ValueError:
-        pass
+    with open(__file__, 'rb') as fo:
+        fastavro.reader(fo)
 
 
 def test_acquaint_schema_rejects_undleclared_name():
@@ -199,6 +199,19 @@ def test_acquaint_schema_accepts_nested_records_from_arrays():
     assert 'Nested' in fastavro._writer.SCHEMA_DEFS
 
 
+def test_compose_schemas():
+    schema_path = join(data_dir, 'Parent.avsc')
+    schema = fastavro.schema.load_schema(schema_path)
+    assert isinstance(schema, dict)
+    assert 'Child' in fastavro._reader.READERS
+
+
+@raises(fastavro.schema.UnknownType)
+def test_missing_schema():
+    schema_path = join(data_dir, 'ParentMissingChild.avsc')
+    fastavro.schema.load_schema(schema_path)
+
+
 def test_schemaless_writer_and_reader():
     schema = {
         "type": "record",
@@ -235,6 +248,29 @@ def test_default_values():
     assert new_records == [{"default_field": "default_value"}]
 
 
+def test_nullable_values():
+    schema = {
+        "type": "record",
+        "fields": [{
+            "name": "nullable_field",
+            "type": ["string", "null"]
+        }, {
+            "name": "field",
+            "type": "string"
+        }
+        ]
+    }
+    new_file = MemoryIO()
+    records = [{"field": "val"}, {"field": "val", "nullable_field": "no_null"}]
+    fastavro.writer(new_file, schema, records)
+    new_file.seek(0)
+    new_reader = fastavro.reader(new_file)
+    new_records = list(new_reader)
+    print(new_records)
+    assert new_records == [{'nullable_field': None, 'field': 'val'}, {
+        'nullable_field': 'no_null', 'field': 'val'}]
+
+
 def test_boolean_roundtrip():
     schema = {
         "type": "record",
@@ -273,6 +309,42 @@ def test_metadata():
     assert new_reader.metadata['key'] == metadata['key']
 
 
+def test_write_union_shortcut():
+    schema = {
+        "type": "record",
+        "name": "A",
+        "fields": [{
+            "name": "a",
+            "type": [
+                {
+                    "type": "record",
+                    "name": "B",
+                    "fields": [{
+                        "name": "b",
+                        "type": "string"
+                    }]
+                },
+                {
+                    "type": "record",
+                    "name": "C",
+                    "fields": [{
+                        "name": "c",
+                        "type": "string"
+                    }]
+                }
+            ]
+        }]
+    }
+
+    new_file = MemoryIO()
+    records = [{"a": ("B", {"b": "test"})}]
+    fastavro.writer(new_file, schema, records)
+    new_file.seek(0)
+    new_reader = fastavro.reader(new_file)
+    new_records = list(new_reader)
+    assert new_records == [{"a": {"b": "test"}}]
+
+
 def test_repo_caching_issue():
     schema = {
         "type": "record",
@@ -522,6 +594,7 @@ def test_schema_migration_reader_union():
     assert new_records == records
 
 
+@raises(fastavro._reader.SchemaResolutionError)
 def test_schema_migration_union_failure():
     schema = {
         "type": "record",
@@ -544,14 +617,11 @@ def test_schema_migration_union_failure():
     fastavro.writer(new_file, schema, records)
     new_file.seek(0)
     new_reader = fastavro.reader(new_file, new_schema)
-    try:
-        list(new_reader)
-    except fastavro._reader.SchemaResolutionError:
-        pass
-    else:
-        assert False
+
+    list(new_reader)
 
 
+@raises(fastavro._reader.SchemaResolutionError)
 def test_schema_migration_array_failure():
     schema = {
         "type": "record",
@@ -580,14 +650,10 @@ def test_schema_migration_array_failure():
     fastavro.writer(new_file, schema, records)
     new_file.seek(0)
     new_reader = fastavro.reader(new_file, new_schema)
-    try:
-        list(new_reader)
-    except fastavro._reader.SchemaResolutionError:
-        pass
-    else:
-        assert False
+    list(new_reader)
 
 
+@raises(fastavro._reader.SchemaResolutionError)
 def test_schema_migration_maps_failure():
     schema = {
         "type": "record",
@@ -616,14 +682,10 @@ def test_schema_migration_maps_failure():
     fastavro.writer(new_file, schema, records)
     new_file.seek(0)
     new_reader = fastavro.reader(new_file, new_schema)
-    try:
-        list(new_reader)
-    except fastavro._reader.SchemaResolutionError:
-        pass
-    else:
-        assert False
+    list(new_reader)
 
 
+@raises(fastavro._reader.SchemaResolutionError)
 def test_schema_migration_enum_failure():
     schema = {
         "type": "enum",
@@ -642,14 +704,10 @@ def test_schema_migration_enum_failure():
     fastavro.writer(new_file, schema, records)
     new_file.seek(0)
     new_reader = fastavro.reader(new_file, new_schema)
-    try:
-        list(new_reader)
-    except fastavro._reader.SchemaResolutionError:
-        pass
-    else:
-        assert False
+    list(new_reader)
 
 
+@raises(fastavro._reader.SchemaResolutionError)
 def test_schema_migration_schema_mismatch():
     schema = {
         "type": "record",
@@ -670,9 +728,48 @@ def test_schema_migration_schema_mismatch():
     fastavro.writer(new_file, schema, records)
     new_file.seek(0)
     new_reader = fastavro.reader(new_file, new_schema)
-    try:
-        list(new_reader)
-    except fastavro._reader.SchemaResolutionError:
-        pass
-    else:
-        assert False
+    list(new_reader)
+
+
+@raises(EOFError)
+def test_empty():
+    io = MemoryIO()
+    schema = {
+        'type': 'record',
+        'name': 'test',
+        'fields': [
+            {'type': 'boolean', 'name': 'a'}
+        ],
+    }
+    fastavro.load(io, schema)
+
+
+@raises(ValueError)
+def test_no_default():
+    io = MemoryIO()
+    schema = {
+        'type': 'record',
+        'name': 'test',
+        'fields': [
+            {'type': 'boolean', 'name': 'a'}
+        ],
+    }
+    fastavro.writer(io, schema, [{}])
+
+
+def test_validator():
+    raise SkipTest('FIXME: Add tests for write validator argument')
+
+
+def test_is_avro_str():
+    for path in iglob('%s/*.avro' % data_dir):
+        assert fastavro.is_avro(path)
+    assert not fastavro.is_avro(__file__)
+
+
+def test_is_avro_fo():
+    for path in iglob('%s/*.avro' % data_dir):
+        with open(path, 'rb') as fp:
+            assert fastavro.is_avro(fp)
+    with open(__file__, 'rb') as fp:
+        assert not fastavro.is_avro(fp)
